import { codeBlock } from 'discord.js';

import { Message } from '../../bot/index.js';
import { parseBody } from '../../utils/command.js';
import { sendWithRetry } from '../../utils/rcon.js';
import { parseTable } from '../../utils/table.js';
import { Command, CommandContext } from '../index.js';

/**
 * Options for recent players command, row limit.
 */
export interface ConanRecentArgs {
  count: number;
}

/**
 * List players who have recently been online. Probably only works for Conan Exiles.
 */
export const conanRecent: Command = {
  name: 'conan-recent',
  desc: 'show the most recently online players',
  async run(context: CommandContext, message: Message) {
    const [_body, args] = await parseBody<ConanRecentArgs>(message.content, conanRecent, context.args.commandPrefix, {
      count: {
        default: 3,
        type: 'number',
      },
    });
    const count = Math.min(args.count, 10);

    context.logger.debug({
      count,
    }, 'getting recent players');

    const players = await sendWithRetry(context.rcon, `sql \
SELECT \
  char_name AS name, \
  level, \
  guilds.name AS guild, \
  datetime(lastTimeOnline, 'unixepoch') AS last_seen \
FROM characters \
INNER JOIN guilds ON characters.guild = guilds.guildId \
ORDER BY lastTimeOnline DESC \
LIMIT ${count}`, context.logger);

    context.logger.debug({
      players: parseTable(players),
    }, 'players table');

    await message.reply(`recent players: ${codeBlock(players)}`);
  },
};
