import { Logger } from 'noicejs';

import { ParsedArgs } from '../args.js';
import { Author, Bot, Message } from '../bot/index.js';
import { DataLayer } from '../data/index.js';
import { RconClient } from '../utils/rcon.js';
import { rconSQL } from './admin/rcon-sql.js';
import { rcon } from './admin/rcon.js';
import { conanOnline } from './conan/online.js';
import { conanRecent } from './conan/recent.js';
import { help } from './help.js';
import { karma } from './karma.js';
import { ping } from './ping.js';

/**
 * Bot command with optional authn/authz.
 *
 * @public
 */
export interface Command {
  desc: string;
  name: string;

  auth?: (context: CommandContext, author: Author) => Promise<boolean>;
  run: (context: CommandContext, message: Message) => Promise<void>;
}

/**
 * Context passed to {@link Command.run} when invoked.
 *
 * @public
 */
export interface CommandContext {
  args: ParsedArgs;
  bot: Bot;
  commands: ReadonlyArray<Command>;
  data: DataLayer;
  logger: Logger;
  rcon: RconClient;
}

/**
 * Available commands.
 *
 * @private
 */
export const COMMANDS: Array<Command> = [
  conanOnline,
  conanRecent,
  help,
  karma,
  ping,
  rcon,
  rconSQL,
];
