import { mustExist } from '@apextoaster/js-utils';
import { gracefulShutdown, scheduleJob } from 'node-schedule';
import { format as sprintf } from 'node:util';
import { Logger } from 'noicejs';

import { ParsedArgs } from '../args.js';
import { Bot } from '../bot/index.js';
import { DataLayer } from '../data/index.js';
import { RconClient } from '../utils/rcon.js';

export interface Job {
  destroy(): Promise<void>;
}

export interface JobContext {
  args: ParsedArgs;
  bot: Bot;
  data: DataLayer;
  logger: Logger;
  rcon: RconClient;
}

export const JOBS: Array<[string, (context: JobContext) => Promise<void>]> = [
  ['*/15 * * * *', playerCountJob],
];

export async function playerCountJob(context: JobContext): Promise<void> {
  const { args, bot, logger, rcon } = context;
  logger.info('player count job');

  const players = await rcon.send('listplayers');
  const count = players.trim().split('\n').length - 1;
  logger.debug({ players, count }, 'player count cron');

  // TODO: use `data` to store value/check for changes

  const channel = mustExist(await bot.getChannel(args.channelId));
  const topic = sprintf(args.channelTopic, count);
  logger.info({ channel, topic }, 'updating channel topic');

//   await channel.edit({
//     topic: `${topic}, players online: ${count}`,
//   });
}

export async function startCrons(context: JobContext): Promise<Job> {
  const { logger } = context;

  for (const [schedule, fn] of JOBS) {
    const job = scheduleJob(fn.name, schedule, () => fn(context));
    logger.debug({
      job: fn.name,
      next: job.nextInvocation(),
    }, 'scheduled cron job');
  }

  return {
    destroy() {
      return gracefulShutdown();
    },
  };
}
