import { expect } from 'chai';
import { NullLogger } from 'noicejs';
import { match, mock } from 'sinon';

import { parseArgs } from '../../src/args.js';
import { Message } from '../../src/bot/index.js';
import { help } from '../../src/command/help.js';
import { CommandContext } from '../../src/command/index.js';
import { MemoryDataLayer } from '../../src/data/memory.js';
import { mockAuthor, mockBot, mockChannel, mockRcon, REQUIRED_ARGS } from '../helpers.js';

describe('help command', () => {
  it('should reply with a list of commands', async () => {
    const args = await parseArgs([
      ...REQUIRED_ARGS,
      '--commandPrefix="!"',
    ]);
    const context: CommandContext = {
      args,
      bot: mockBot({}),
      commands: [help],
      data: new MemoryDataLayer(args),
      logger: NullLogger.global,
      rcon: mockRcon(),
    };
    const replyMock = mock().named('Message.reply').resolves();
    const message: Message = {
      author: mockAuthor(),
      channel: mockChannel(),
      content: '',
      reply: replyMock,
    };

    await help.run(context, message);

    expect(replyMock).to.have.callCount(1).and.been.calledWith(match(`!${help.name}: ${help.desc}`));
  });
});
