import { expect } from 'chai';
import { NullLogger } from 'noicejs';
import { mock } from 'sinon';

import { parseArgs } from '../../src/args.js';
import { Message } from '../../src/bot/index.js';
import { CommandContext } from '../../src/command/index.js';
import { ping } from '../../src/command/ping.js';
import { MemoryDataLayer } from '../../src/data/memory.js';
import { mockAuthor, mockChannel, mockRcon, REQUIRED_ARGS } from '../helpers.js';

describe('ping command', () => {
  it('should reply with a pong', async () => {
    const author = mockAuthor();
    const channel = mockChannel();
    const args = await parseArgs([
      ...REQUIRED_ARGS,
      '--commandPrefix="!"',
    ]);
    const context: CommandContext = {
      args,
      bot: {
        connect: () => Promise.resolve(true),
        destroy: () => { /* noop */ },
        getChannel: mock().atLeast(1).resolves(channel),
        getUser: mock().atLeast(1).resolves(author),
      },
      commands: [],
      data: new MemoryDataLayer(args),
      logger: NullLogger.global,
      rcon: mockRcon(),
    };

    const replyMock = mock().named('Message.reply').resolves();
    const message: Message = {
      author,
      channel,
      content: '',
      reply: replyMock,
    };

    await ping.run(context, message);

    expect(replyMock).to.have.callCount(1);
  });
});
