import { expect } from 'chai';

import { Command } from '../../src/command/index.js';
import { matchCommands, parseBody, removeCommandName } from '../../src/utils/command.js';

const TEST_COMMANDS: Array<Command> = [{
  name: 'foo',
  desc: 'bar',
  run: async () => { /* noop */ },
}, {
  name: 'hello',
  desc: 'world',
  run: async () => { /* noop */ },
}];

describe('command utils', () => {
  describe('author name helper', () => {
    it('should append their discriminator');
  });

  describe('command name helper', () => {
    it('should prepend the prefix');
  });

  describe('match commands helper', () => {
    it('should return commands whose name matches the first word', async () => {
      const matches = matchCommands('!foo bar', TEST_COMMANDS, '!');
      expect(matches.length).to.equal(1);
    });
  });

  describe('remove command name helper', () => {
    it('should remove the command name from the message if it matches', async () => {
      expect(removeCommandName('!foo bar', TEST_COMMANDS[0], '!')).to.equal('bar');
      expect(removeCommandName('!foo bar', TEST_COMMANDS[1], '!')).to.equal('!foo bar');
    });
  });

  describe('parse body helper', () => {
    it('should parse dashed options from the message', async () => {
      interface TestArgs {
        count: number;
      };

      const [_body, args] = await parseBody<TestArgs>('!foo --count 3', TEST_COMMANDS[0], '!', {
        count: {
          default: 0,
          type: 'number',
        },
      });

      // eslint-disable-next-line @typescript-eslint/no-magic-numbers
      expect(args.count).to.equal(3);
    });

    it('should parse positional arguments from the message');

    it('should return the remainder of the message', async () => {
      interface TestArgs {
        count: number;
      };

      const [body, _args] = await parseBody<TestArgs>('!foo --count 3 bar', TEST_COMMANDS[0], '!', {
        count: {
          default: 0,
          type: 'number',
        },
      });

      expect(body).to.equal('bar');
    });
  });
});
